@@ -100,6 +100,14 @@ class DTestBase(object):
     DTest class to return True.
     """
 
+    # Keep a list of the recognized attributes for the promote()
+    # classmethod
+    _class_attributes = [
+        '_name', '_test', '_class', '_exp_fail', '_skip', '_pre', '_post',
+        '_deps', '_revdeps', '_partner', '_attrs', '_raises', '_timeout',
+        '_result',
+        ]
+
     def __init__(self, test):
         """
         Initialize a DTestBase instance wrapping ``test``.
@@ -112,7 +120,21 @@ class DTestBase(object):
         # We have to unwrap MethodType and class and static methods
         if (isinstance(test, types.MethodType) or
             isinstance(test, classmethod) or isinstance(test, staticmethod)):
-            test = test.__func__
+            try:
+                test = test.__func__
+            except AttributeError:
+                # Python 2.6 doesn't have __func__ attribute on
+                # classmethod or staticmethod, so let's kludge around
+                # it...
+                tmp = test.__get__(None, object)
+
+                # If it's an instance of staticmethod, tmp is func
+                if isinstance(test, staticmethod):
+                    test = tmp
+
+                # If it's an instance of classmethod, tmp has __func__
+                else:
+                    test = tmp.__func__
 
         # Require it to be a callable...
         if not callable(test):
@@ -365,6 +387,44 @@ class DTestBase(object):
         # an attribute
         return self._timeout
 
+    @classmethod
+    def promote(cls, test):
+        """
+        Promotes a DTestBase instance from one class to another.
+        """
+
+        # If test is None, return None
+        if test is None:
+            return None
+
+        # If it's already the same class, return it
+        if test.__class__ == cls:
+            return test
+
+        # First, allocate a new instance
+        newtest = object.__new__(cls)
+
+        # Now, initialize it from test
+        for attr in cls._class_attributes:
+            setattr(newtest, attr, getattr(test, attr))
+
+        # Walk through all dependencies/dependents and replace the
+        # test
+        for dep in newtest._deps:
+            dep._revdeps.remove(test)
+            dep._revdeps.add(newtest)
+        for dep in newtest._revdeps:
+            dep._deps.remove(test)
+            dep._deps.add(newtest)
+
+        # Replace the bindings in the test
+        newtest._test._dt_dtest = newtest
+        newtest._test.setUp = newtest.setUp
+        newtest._test.tearDown = newtest.tearDown
+
+        # Return the new test
+        return newtest
+
     def setUp(self, pre):
         """
         Explicitly set the setUp() function or method to be called
@@ -476,13 +536,27 @@ class DTestBase(object):
     def _depcheck(self, output):
         """
         Performs a check of all this test's dependencies, to determine
-        if the test can be executed.  This is an abstract method that
-        is overridden by the DTest and DTestFixture classes to
-        implement class-specific behavior.
+        if the test can be executed.  Tests can only be executed if
+        all their dependencies have passed.
         """
 
-        # Abstract method; subclasses must define!
-        raise Exception("Subclasses must implement _depcheck()")
+        # All dependencies must be OK
+        for dep in self._deps:
+            if (dep.state == FAIL or dep.state == ERROR or
+                dep.state == XFAIL or dep.state == DEPFAIL):
+                # Set our own state to DEPFAIL
+                self._result._transition(DEPFAIL, output=output)
+                return False
+            elif dep.state == SKIPPED:
+                # Set our own state to SKIPPED
+                self._result._transition(SKIPPED, output=output)
+                return False
+            elif dep.state != OK and dep.state != UOK:
+                # Dependencies haven't finished up, yet
+                return False
+
+        # All dependencies satisfied!
+        return True
 
     def _skipped(self, output):
         """
@@ -547,31 +621,6 @@ class DTest(DTestBase):
         # count
         return 1
 
-    def _depcheck(self, output):
-        """
-        Performs a check of all this test's dependencies, to determine
-        if the test can be executed.  Tests can only be executed if
-        all their dependencies have passed.
-        """
-
-        # All dependencies must be OK
-        for dep in self._deps:
-            if (dep.state == FAIL or dep.state == ERROR or
-                dep.state == XFAIL or dep.state == DEPFAIL):
-                # Set our own state to DEPFAIL
-                self._result._transition(DEPFAIL, output=output)
-                return False
-            elif dep.state == SKIPPED:
-                # Set our own state to SKIPPED
-                self._result._transition(SKIPPED, output=output)
-                return False
-            elif dep.state != OK and dep.state != UOK:
-                # Dependencies haven't finished up, yet
-                return False
-
-        # All dependencies satisfied!
-        return True
-
     def istest(self):
         """
         Returns True if the instance is a test or False if the
@@ -615,40 +664,6 @@ class DTestFixture(DTestBase):
         # Now, save our pair partner
         self._partner = setUp
 
-    def _depcheck(self, output):
-        """
-        Performs a check of all this test fixture's dependencies, to
-        determine if the test can be executed.  Test fixtures can only
-        be executed if their partner (if one is specified) has passed
-        and if all the tests the fixture is dependent on have finished
-        running or been skipped.
-        """
-
-        # Make sure our partner succeeded
-        if self._partner is not None:
-            if (self._partner.state == FAIL or
-                self._partner.state == XFAIL or
-                self._partner.state == ERROR or
-                self._partner.state == DEPFAIL):
-                # Set our own state to DEPFAIL
-                self._result._transition(DEPFAIL, output=output)
-                return False
-            elif self._partner.state == SKIPPED:
-                # Set our own state to SKIPPED
-                self._result._transition(SKIPPED, output=output)
-                return False
-
-        # Other dependencies must not be un-run or in the RUNNING
-        # state
-        for dep in self._deps:
-            if dep.state is None or dep.state == RUNNING:
-                return False
-
-        # Dependencies can have failed, failed due to dependencies,
-        # been skipped, or have completed--they just have to be in
-        # that state before running the fixture
-        return True
-
     def _skipped(self, output):
         """
         Marks this DTestFixture instance as having been skipped.  Test
@@ -683,7 +698,68 @@ class DTestFixture(DTestBase):
         super(DTestFixture, self)._skipped(output)
 
 
-def _gettest(func, testcls=DTest):
+class DTestFixtureSetUp(DTestFixture):
+    """
+    DTestFixtureSetUp
+    =================
+
+    The DTestFixtureSetUp class represents setUp() and setUpClass()
+    test fixtures to be executed before enclosed tests.  It is derived
+    from DTestFixture.
+    """
+
+    pass
+
+
+class DTestFixtureTearDown(DTestFixture):
+    """
+    DTestFixtureTearDown
+    ====================
+
+    The DTestFixtureTearDown class represents tearDown() and
+    tearDownClass() test fixtures to be executed after enclosed tests.
+    It is derived from DTestFixture, but overrides the _depcheck()
+    method to ensure that tearDown() and tearDownClass() are always
+    called even if some of the dependencies have failed (unless the
+    corresponding setUp() or setUpClass() fixtures have failed).
+    """
+
+    def _depcheck(self, output):
+        """
+        Performs a check of all this test fixture's dependencies, to
+        determine if the test can be executed.  Test fixtures can only
+        be executed if their partner (if one is specified) has passed
+        and if all the tests the fixture is dependent on have finished
+        running or been skipped.
+        """
+
+        # Make sure our partner succeeded
+        if self._partner is not None:
+            if (self._partner.state == FAIL or
+                self._partner.state == XFAIL or
+                self._partner.state == ERROR or
+                self._partner.state == DEPFAIL):
+                # Set our own state to DEPFAIL
+                self._result._transition(DEPFAIL, output=output)
+                return False
+            elif self._partner.state == SKIPPED:
+                # Set our own state to SKIPPED
+                self._result._transition(SKIPPED, output=output)
+                return False
+
+        # Other dependencies must not be un-run or in the RUNNING
+        # state
+        for dep in self._deps:
+            if dep.state is None or dep.state == RUNNING:
+                return False
+
+        # Dependencies can have failed, failed due to dependencies,
+        # been skipped, or have completed--they just have to be in
+        # that state before running the fixture
+        return True
+
+
+def _gettest(func, testcls=DTest, promote=False):
     """
     Retrieves a DTest from--or, if ``testcls`` is not None, attaches a
     new test of that class to--``func``.  This is a helper function
@@ -692,11 +768,25 @@ def _gettest(func, testcls=DTest):
 
     # We could be passed a DTest, so return it if so
     if isinstance(func, DTestBase):
-        return func
+        return testcls.promote(func) if promote else func
 
     # If it's a class method or static method, unwrap it
     if isinstance(func, (classmethod, staticmethod)):
-        func = func.__func__
+        try:
+            func = func.__func__
+        except AttributeError:
+            # Python 2.6 doesn't have __func__ attribute on
+            # classmethod or staticmethod, so let's kludge around
+            # it...
+            tmp = func.__get__(None, object)
+
+            # If it's an instance of staticmethod, tmp is func
+            if isinstance(func, staticmethod):
+                func = tmp
+
+            # If it's an instance of classmethod, tmp has __func__
+            else:
+                func = tmp.__func__
 
     # Always return None if _dt_nottest is set
     if func is None or (hasattr(func, '_dt_nottest') and func._dt_nottest):
@@ -704,7 +794,7 @@ def _gettest(func, testcls=DTest):
 
     # Look up the test as a function attribute
     try:
-        return func._dt_dtest
+        return testcls.promote(func._dt_dtest) if promote else func._dt_dtest
     except AttributeError:
         # Don't want to create one, I guess
         if testcls is None:
@@ -749,13 +839,10 @@ def isfixture(func):
     """
     Decorates a function to indicate that the function is a test
     fixture, i.e., package- or module-level setUp()/tearDown() or
-    class-level setUpClass()/tearDownClass().  This decorator must be
-    used first on any fixture that one of the other decorators (e.g.,
-    @depends(), @timed()) will be used on.
-    """
+    class-level setUpClass()/tearDownClass().
 
-    # Make sure func has a DTestFixture associated with it
-    _gettest(func, DTestFixture)
+    This decorator is now deprecated.
+    """
 
     # Return the function
     return func
@@ -932,7 +1019,8 @@ def visit_mod(mod, tests):
     # See if we have fixtures in this module
     setUpLocal = None
     if hasattr(mod, SETUP):
-        setUpLocal = _gettest(getattr(mod, SETUP), DTestFixture)
+        setUpLocal = _gettest(getattr(mod, SETUP),
+                              DTestFixtureSetUp, True)
 
         # Set up the dependency
         if setUp is not None:
@@ -940,7 +1028,8 @@ def visit_mod(mod, tests):
 
         setUp = setUpLocal
     if hasattr(mod, TEARDOWN):
-        tearDownLocal = _gettest(getattr(mod, TEARDOWN), DTestFixture)
+        tearDownLocal = _gettest(getattr(mod, TEARDOWN),
+                                 DTestFixtureTearDown, True)
 
         # Set up the dependency
         if tearDown is not None:
@@ -1060,9 +1149,9 @@ class DTestCaseMeta(type):
         setUp = getattr(cls, SETUP, None)
         tearDown = getattr(cls, TEARDOWN, None)
         setUpClass = _gettest(getattr(cls, SETUP + CLASS, None),
-                              DTestFixture)
+                              DTestFixtureSetUp, True)
         tearDownClass = _gettest(getattr(cls, TEARDOWN + CLASS, None),
-                                 DTestFixture)
+                                 DTestFixtureTearDown, True)
 
         # Attach the class to the fixtures
         if setUpClass is not None:
@@ -1146,66 +1235,3 @@ class DTestCase(object):
     """
 
     __metaclass__ = DTestCaseMeta
-
-
-def dot(tests, grname='testdeps'):
-    """
-    Constructs a GraphViz-compatible dependency graph with the given
-    name (``testdeps``, by default).  Returns the graph as a string.
-    The graph can be fed to the ``dot`` tool to generate a
-    visualization of the dependency graph.  Note that red nodes in the
-    graph indicate test fixtures, and red dashed edges indicate
-    dependencies associated with test fixtures.  If the node outline
-    is dotted, that indicates that the test was skipped in the most
-    recent test run.
-    """
-
-    # Helper to generate node and edge options
-    def mkopts(opts):
-        # If there are no options, return an empty string
-        if not opts:
-            return ''
-
-        # OK, let's do this...
-        return ' [' + ','.join(['%s="%s"' % (k, opts[k])
-                                for k in opts]) + ']'
-
-    # Now, create the graph
-    nodes = []
-    edges = []
-    for dt in tests:
-        # If it's not a callable, must be class or static method;
-        # get the real test
-        test = dt._test if callable(dt._test) else dt._test.__func__
-
-        # Make the node
-        opts = dict(label=r'%s\n%s:%d' %
-                    (dt, test.func_code.co_filename,
-                     test.func_code.co_firstlineno))
-        if dt.state:
-            opts['label'] += r'\n(Result: %s)' % dt.state
-        if (dt.state == FAIL or dt.state == XFAIL or dt.state == ERROR or
-            dt.state == DEPFAIL):
-            opts['color'] = 'red'
-        elif isinstance(dt, DTestFixture):
-            opts['color'] = 'blue'
-        if dt.state == SKIPPED:
-            opts['style'] = 'dotted'
-        elif dt.state == DEPFAIL:
-            opts['style'] = 'dashed'
-        nodes.append('"%s"%s;' % (dt, mkopts(opts)))
-
-        # Make all the edges
-        for dep in dt._deps:
-            opts = {}
-            if (isinstance(dt, DTestFixture) or
-                isinstance(dep, DTestFixture)):
-                opts.update(dict(color='blue', style='dashed'))
-            if dt._partner is not None and dep == dt._partner:
-                opts['style'] = 'dotted'
-
-            edges.append('"%s" -> "%s"%s;' % (dt, dep, mkopts(opts)))
-
-    # Return a graph
-    return (('strict digraph "%s" {\n\t' % grname) +
-            '\n\t'.join(nodes) + '\n\n\t' + '\n\t'.join(edges) + '\n}')
